# auto_home.tcl - format for auto_home table
#
# version -
#	$Id$

set t_width(auto_home) 78

set t_header(auto_home) \
"/home/              source"

proc t_entry_proc(auto_home) {entry} {
 set key [lindex $entry 0]
 set val [lindex $entry 1]
 return [format "%-18s  %-58s" $key $val]
}

proc t_sort_proc(auto_home) {e1 e2} {
  return [expr {[lindex $e1 0] < [lindex $e2 0]}]
}
# --- last line of auto_home.tcl ---
